<?php

/*******************************************************************************
       __        __
      / /   ____/ /___ _____     _________ ___
     / /   / __  / __ `/ __ \   / ___/ __ `__ \
    / /___/ /_/ / /_/ / /_/ /  / /__/ / / / / /
   /_____/\__,_/\__,_/ .___/   \___/_/ /_/ /_/
                    /_/  LDAP Contact Manager for Drupal

      Copyright (c) 2010 - Samuele ~redShadow~ Santi - Under GPL v3

********************************************************************************
  Displaying functions

  Search path description:
    ldap-addressbook/<dir_id>/<filter>
      _GET['order'] -> the ordering filter(s)
    _GET['viewtype'] -> the theming template to use

*******************************************************************************/


/**
 * LDAP AddressBook search form.
 *
 * Parameters to perform the search are read from the drupal path, and
 * from _GET. The form submits via-post to a function that cleans/rewrites
 * the URL then redirects to this page.
 *
 * The form also performes a search, then stores the results as serialized
 * array in a 'value' field -> $form['results'].
 * The theming function should read them and render a table below the form.
 *
 * TODO add support for pager (paged query + themed pager)
 * TODO put full-text search fields definition on directry configuration
 * TODO create a way to define custom fields / widgets / etc.
 *
 * @param array $form_state Passed by Drupal form constructor
 * @param int $dir_id The id of the LDAP directory
 * @param string $fulltext Text to be searched on a list of specified fields
 * @return array The form structure to be themed
 */
function ldapab_search_form($form_state, $dir_id = NULL, $fulltext = NULL) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  if (!user_access('ldapab query addressbook')) {
    drupal_access_denied();
    return;
  }

  // -- Load directory id/object -----------------------------------------------
  $dir_id = (int)$dir_id;
  $directory = ldapab_get_directory($dir_id);
  if (!$directory) {
    drupal_set_message(t("<strong>WARNING!</strong> Directory specified by url doesn't exist!"), 'warning');
    $directory = new stdClass();
  }
  else {
    drupal_set_title(t("Search directory @name", array('@name' => $directory->title)));
  }

  // -- Build filters ----------------------------------------------------------
  // Load fulltext from _GET if present
  if ($_GET['fulltext']) {
    $fulltext = $_GET['fulltext'];
  }

  if ($fulltext) {
    // create fulltext search filter
    $ldap_filter = ldapab_create_filter_fulltext($dir_id, $fulltext);
  }
  elseif ($_GET['filters']) {
    // create filters from detailed attributes
    $filters_val = $_GET['filters'];
    $ldap_filter = ldabap_create_filter_from_form($filters_val, 'and');
  }
  else {
    // do not perform search
    $ldap_filter = NULL;
  }

  // -- Set the breadcrumb -----------------------------------------------------
  drupal_set_breadcrumb(array(
    l(t("Home"), '<front>'),
    l(t("Addressbook"), 'ldapcm'),
    l($directory->title, 'ldapcm/'. $dir_id),
    l(t("Search"), 'ldapcm/'. $dir_id .'/search'),
  ));

  // -- Initialize the form object ---------------------------------------------
  $form = array(
    //'#method' => 'GET'
  );

  $form['dir_id'] = array(
    '#type' => 'value',
    '#value' => $dir_id,
  );

  // -- Administrative links on top of the form --------------------------------
  $form['top_links'] = array(
    '#weight' => -15,
  );
  if ($dir_id && user_access("ldapab manage directories")) {
    $form['top_links'][] = array(
      '#value' => l(t("Admin:Show"), "admin/settings/ldapcm/directories/". $dir_id),
    );
    $form['top_links'][] = array(
      '#value' => l(t("Admin:Edit"), "admin/settings/ldapcm/directories/". $dir_id ."/edit"),
    );
  }

  // -- Create search form -----------------------------------------------------
  $form['search_form_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t("Search"),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // -- Directory selection field ----------------------------------------------
  $form['search_form_wrapper']['dir_id_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t("Directory selection"),
    '#collapsible' => TRUE,
    '#collapsed' => is_null($directory) ? FALSE : TRUE,
    '#weight' => -10,
  );
  $form['search_form_wrapper']['dir_id_wrapper']['dir_id'] = array(
    '#type' => 'select',
    '#title' => t("Directory"),
    '#options' => array(),
    '#description' => t("Select the directory on wich to perform the query"),
    '#default_value' => $dir_id,
  );
  $dirs = ldapab_get_all_directories();
  foreach ($dirs as $id => $dir) {
    $form['dir_id_wrapper']['dir_id']['#options'][$id] = $dir->title;
  }

  // Fields used to set filters
  $search_fields = ldapab_get_searchable_fields();

  // Available filter types
  $filter_types = ldapab_get_filter_types();

  // Full text search in the directory
  $form['search_form_wrapper']['fulltext_search'] = array(
    '#type' => 'fieldset',
    '#title' => t("Full Text Search"),
    '#collapsible' => TRUE,
    '#collapsed' => ($fulltext || !isset($_GET['filters'])) ? FALSE : TRUE,
    '#weight' => -8,
  );
  $form['search_form_wrapper']['fulltext_search']['fulltext'] = array(
    '#type' => 'textfield',
    '#title' => t("Text to search"),
    '#description' => t("Insert here text to be searched in the !fields fields.<br /><strong>ATTENTION!</strong> This will disable the additional filters below.", array('!fields' => "cn,gn,sn,mail")),
    '#weight' => 0,
    '#default_value' => $fulltext,
  );
//  $form['search_form_wrapper']['fulltext_search']['fulltext_fields'] = array(
//    '#type' => 'textfield',
//    '#title' => t("Full text search fields"),
//    '#description' => t("Insert here the list of fields on which to perform the fulltext search.<br />Defaults to <code>!fields</code>.", array('!fields' => "cn,gn,sn,mail")),
//    '#weight' => 1,
//    '#default_value' => $_GET['fulltext_fields'] ? $_GET['fulltext_fields'] : "cn,gn,sn,mail",
//  );

  // Filters "tree" field
  $form['search_form_wrapper']['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t("Advanced filters"),
    '#tree' => TRUE,
    '#collapsible' => TRUE,
    '#collapsed' => (isset($_GET['filters']) && !$fulltext) ? FALSE : TRUE,
    '#weight' => -2,
  );

  foreach ($search_fields as $name => $label) {
    $form['search_form_wrapper']['filters'][$name] = array(
      '#type' => 'fieldset',
      '#title' => $label,
      '#collapsible' => TRUE,
    );
    $form['search_form_wrapper']['filters'][$name]['attr_name'] = array(
      '#type' => 'value',
      '#value' => $name,
      '#default_value' => $filters_val[$name]['attr_name'],
    );
    $form['search_form_wrapper']['filters'][$name]['attr_label'] = array(
      '#type' => 'value',
      '#value' => $label,
    );
    $form['search_form_wrapper']['filters'][$name]['match'] = array(
      '#type' => 'select',
      '#title' => t("Match"),
      '#options' => $filter_types,
      '#default_value' => $filters_val[$name]['match'],
    );
    $form['search_form_wrapper']['filters'][$name]['value'] = array(
      '#type' => 'textfield',
      '#title' => t("Value"),
      '#default_value' => $filters_val[$name]['value'],
    );
  }

  $form['search_form_wrapper']['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Search"),
    '#weight' => 20,
  );

  // If a filter is set, perform search and store results into $form['results']
  if ($ldap_filter) {
    // perform search
    $search = ldapab_search_entries($dir_id, $ldap_filter);

    // append entries to results
    if ($search) {
      $entries = $search->sorted(array('sn', 'givenName'), SORT_ASC);
      $form['results'] = array(
        '#type' => 'value',
        '#value' => serialize($entries),
      );
    }

    //dpm("Search filter: ". $ldap_filter->asString());

  }
  return $form;
}

/**
 * Search submit handler
 * @param array $form
 * @param array $form_state
 * @return void
 * TODO call some function from ldapab.ldap.inc to build filters!
 */
function ldapab_search_form_submit($form, &$form_state) {
  if (!user_access('ldapab query addressbook')) {
    drupal_access_denied();
    return;
  }

  // Build the URL
  $values = $form_state['values'];
  $newquery = array();
  $newurl = "ldapcm/". (int)$values['dir_id'] ."/search";

  if ($values['fulltext']) {
    // Append fulltext value as inline
    $newurl .= "/". urlencode($values['fulltext']);
  }
  else {
    // Build GET query for the filters
    foreach ($values['filters'] as $filter) {
      if ($filter['match']) {
        $newquery['filters'][$filter['attr_name']] = array(
          'attr_name' => $filter['attr_name'],
          'match' => $filter['match'],
          'value' => $filter['value'],
        );
      }
    }
  }

  $destination = preg_replace("#^/#", "", url($newurl, array('query' => $newquery)));
  $form_state['redirect'] = $destination;

  return;
}


/**
 * Themeize the search form
 * @param array $form
 * @return string
 */
function theme_ldapab_search_form($form) {
  $header = NULL;
  $rows = array();

  $dir_id = $form['dir_id']['#value'];

  $toplinks = array();
  foreach (element_children($form['top_links']) as $tl) {
    $toplinks[] = drupal_render($form['top_links'][$tl]);
  }
  $form['top_links'] = array(
    '#weight' => -15,
    '#prefix' => "<div style='text-align: right;'>",
    '#suffix' => "</div>",
    '#value' => implode(" | ", $toplinks)
  );

  foreach (element_children($form['search_form_wrapper']['filters']) as $name) {
    unset($form['search_form_wrapper']['filters'][$name]['match']['#title']);
    unset($form['search_form_wrapper']['filters'][$name]['value']['#title']);
    $rows[] = array(
      array('data' => $form['search_form_wrapper']['filters'][$name]['attr_label']['#value'], 'header' => TRUE),
      drupal_render($form['search_form_wrapper']['filters'][$name]['match']),
      drupal_render($form['search_form_wrapper']['filters'][$name]['value']),
    );
    unset($form['search_form_wrapper']['filters'][$name]);
  }

  $form['search_form_wrapper']['filters']['#value'] = theme('table', $header, $rows);

  $entries = unserialize($form['results']['#value']);

  if (is_array($entries)) {
    $form['results'] = array(
      '#type' => 'fieldset',
      '#title' => t("Search results"),
      '#value' => "<div>no search performed</div>",
      '#weight' => 20,
      '#collapsible' => TRUE,
    );

    $header = array('uid', 'Name', 'Surname', 'E-Mail', t("Actions"));
    $rows = array();
    foreach ($entries as $dn => $entry) {
    $rows[] = array(
      //$entry->dn(),
      $entry->exists('uid') ? $entry->getValue('uid', 'single') : $entry->dn(),
      $entry->exists('givenName') ? $entry->getValue('givenName', 'single') : "---",
      $entry->exists('sn') ? $entry->getValue('sn', 'single') : "---",
      $entry->exists('mail') ? theme('item_list', $entry->getValue('mail', 'all')) : "---",
      l(t("Show"), 'ldapcm/'. $dir_id .'/entry/'. urlencode($entry->dn()))
        .' '. l(t("Edit"), 'ldapcm/'. $dir_id .'/entry/'. urlencode($entry->dn()) .'/edit'),
      );
    }

    $form['results']['#value'] = theme('table', $header, $rows);
  }
  else {
    unset($form['results']);
  }

  return drupal_render($form);
}


/**
 * List available directories
 * @return string
 */
function ldapab_page_listdirs() {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  $directories = ldapab_get_all_directories();
  $header = array(t("Name"), t("Host"), t("Bind DN"), t("Actions"));
  $rows = array();
  foreach ($directories as $id => $dir) {
    $rows[] = array(
      l($dir->title, 'ldapcm/'. $dir->id),
      $dir->connection['host'],
      $dir->connection['binddn'],
      l(t("Show"), 'ldapcm/'. $dir->id)
      .' '. l(t("Query"), 'ldapcm/'. $dir->id .'/search')
    );
  }
  return theme('table', $header, $rows);
}


/**
 * Show directory informations
 * @return string
 */
function ldapab_page_showdir($dir_id) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  $dir = ldapab_get_directory($dir_id);
  drupal_set_title(t("Directory: @name", array('@name' => $dir->title)));
  $description = str_replace("\n", "<br />\n", $dir->description);
  return "$description";
}


/**
 * Show an entry
 * @param $dir_id
 * @param $entry_dn
 * @return unknown_type
 * TODO call a theming function (and maybe a template too) to create this page
 */
function ldapab_page_showentry($dir_id, $entry_dn) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  $dir = ldapab_get_directory($dir_id);
  $entry = ldapab_get_entry($dir_id, $entry_dn);
  $schema = ldapab_get_fields_schema($dir_id, $entry_dn);
  $rows = array();
  $rows[] = array(
    array('data' => t("Directory"), 'header' => TRUE),
    $dir->title ." (". $dir_id .")",
  );
  $rows[] = array(
    array('data' => t("dn"), 'header' => TRUE),
    $entry->dn(),
  );
  
  if ($entry->exists('cn')) {
    drupal_set_title($entry->getValue('cn', 'single'));
  }
  
  $fields = $entry->attributes();
  
  foreach ($fields as $field_name) {
    if ($entry->exists($field_name)) {
      if (isset($schema['fields'][$field_name]['title'])) {
        $field_title = $schema['fields'][$field_name]['title'];
      }
      else {
        $field_title = $field_name;
      }
      $rows[] = array(
        array('data' => $field_title, 'header' => TRUE),
        implode("<br />", $entry->getValue($field_name, 'all')),
      );
    }
  }

  //drupal_set_title(t("Directory: @name", array('@name' => $dir->title)));
  
  //"[DIR:$dir_id] ENTRY $entry_dn"
  return theme('table', NULL, $rows);
}

/**
 * Page to show the schema of a LDAP entry
 * @param int $dir_id
 * @param string $entry_dn
 * @return string
 */
function ldapab_page_directory_schema($dir_id) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  $schema = ldapab_get_directory_schema($dir_id, $entry_dn);
  
  $out = "";
  
  $out .= "<h3>". t("objectClasses") ."</h3>";
  $objclasses = $schema->getAll("objectClasses");
  //$out .= kpr($objclasses, TRUE);
  
  $found_oc_keys = array();
  
  /*

      ObjectClasses keys:
        name -> string
        desc -> string
        
        aliases -> string[]
        
        must -> string[]
        may -> string[]
        
        sup
        type
        
        oid -> string
        
        abstract -> flag
        auxiliary -> flag
        structural -> flag

   */
  
  // List all the found objectClasses
  $html_oc = array();
  foreach ($objclasses as $id => $def) {
    $oc_name = $def['name'];
    $oc_title = t("Object Class") .": ". $oc_name ." (". $id .")";
    $oc_val = "";
    
    $oc_val .= "<div class='description'>". $def['desc'] ."</div>";
    
    // objectClass aliases
    if ($def['aliases']) {
      $oc_val .= "<div><strong>".t("Aliases").":</strong> ".implode(', ', $def['aliases'])."</div>";
    }
    
    // OID
    if ($def['oid']) {
      $oc_val .= "<div><strong>".t("OID").":</strong> ". $def['oid'] ."</div>";
    }
    
    // Parent objectClasses
    if ($def['sup']) {
      $sup_items = array();
      foreach ($def['sup'] as $sup) {
        $sup_items[] = "<a href='#objectclass-$sup'>$sup</a>";
      }
      $oc_val .= theme('fieldset', array(
        '#title' => t("Parents"),
        '#collapsible' => TRUE,
        '#value' => theme('item_list', $sup_items),
      ));
    }
    
    // Required attributes list
    if ($def['must']) {
      $must_items = array();
      foreach ($def['must'] as $must) {
        $must_items[] = "<a href='#attribute-$must'>$must</a>";
      }
      $oc_val .= theme('fieldset', array(
        '#title' => t("Required Attributes"),
        '#collapsible' => TRUE,
        '#value' => theme('item_list', $must_items),
      ));
    }
    
    // Optional attributes list
    if ($def['may']) {
      $may_items = array();
      foreach ($def['may'] as $may) {
        $may_items[] = "<a href='#attribute-$may'>$may</a>";
      }
      $oc_val .= theme('fieldset', array(
        '#title' => t("Optional Attributes"),
        '#collapsible' => TRUE,
        '#value' => theme('item_list', $may_items),
      ));
    }
    
    unset($def['name'], $def['oid'], $def['desc'], $def['aliases'], $def['may'], $def['must'], $def['sup']);
    
    $rows = array();
    foreach ($def as $key => $value) {
      $found_oc_keys[] = $key;
      $rows[] = array(
        array('data' => $key, 'header' => TRUE),
        array('data' => is_string($value) ? $value : (is_array($value) ? theme('item_list', $value) : kpr($value, TRUE))),
      );
    }
    
    $oc_val .= theme('table', NULL, $rows);
    
    $html_oc[$id] = "<a name='objectclass-$oc_name'></a>" . theme('fieldset', array(
      '#title' => $oc_title,
      '#value' => $oc_val,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ));
    
  }
  ksort($html_oc);
  $out .= implode('', $html_oc);
  
  $found_oc_keys = array_unique($found_oc_keys);
  dpm($found_oc_keys);
  
  //dpm($schema->types);
  
  foreach (array('objectclasses', 'attributes', 'ditcontentrules',
    'ditstructurerules', 'matchingrules', 'matchingruleuses', 'nameforms',
    'syntaxes') as $type) {
    $out .= "<h3>". $type ."</h3>";
    $out .= kpr($schema->getAll($type), TRUE);
  }
  
  return $out . "schema";
}

